/*=====================================================================================================================

   " ".

      .

=======================================================================================================================

  :

  25.06.2015 [*] ExeElementMoveDocument:    NameTrademark.
  29.04.2015 [*] ExeElementMoveDocument:   .
  28.02.2015 [*] ExeElementMoveDocument:    .
             [+] RepairElementModelKind:    ModelKind.
  25.02.2015 [*] ExeElementMoveDocument:    .
  31.07.2012 [*] DisplaceDevice:       .
  15.07.2012 [*]   UPDATE    . 
  14.07.2012 [*] ExeElementMoveDocument:      .
  27.03.2012 [*] DisplaceWorkplace:      
  22.03.2012 [+] DisplaceDepartment:     Displacement.
             [+] DisplaceWorkplace:      Displacement.
  21.03.2012 [+] DisplaceDevice:     Displacement.
             [-] CompleteSetMoving:   DisplaceHard.
  01.03.2012 [+] ExeElementMoveDocument:      .
             [+]  .
  09.10.1997 [!]  .

=======================================================================================================================
  Copyright 19972015 TSK Developer Laboratory, Moscow.                                         Konstantin Tsvetkov
                                                                                                  e-mail: dev@tsklab.ru
                                                                                                  http://www.tsklab.ru
  CVS: $Id: scElementMove.sql,v 1.9 2015/07/22 11:12:39 tsk Exp $
=====================================================================================================================*/
USE ComputerWare
--
EXECUTE CheckDropProcedure  'ExeElementMoveDocument'
EXECUTE CheckDropProcedure  'RepairElementModelKind'
EXECUTE CheckDropTrigger    'DisplaceHard'
EXECUTE CheckDropTrigger    'DisplaceDevice'
EXECUTE CheckDropTrigger    'DisplaceWorkplace'
EXECUTE CheckDropTrigger    'DisplaceDepartment'
--
GO
--=====================================================================================================================
CREATE PROCEDURE ExeElementMoveDocument @EL INT
AS
  SET NOCOUNT ON
  --
  UPDATE CompleteSet
    SET CmplNumber = LEFT(CONVERT(VARCHAR, ISNULL([Document].DocDate, CompleteSet.CmplDateSet), 120), 10) 
                                                                                              + '-' + CompleteSet.HEXID
    FROM CompleteSet 
      LEFT OUTER JOIN [Document] ON CompleteSet.DocumentId = [Document].IdDocument
    WHERE (CompleteSet.IdComponent = @EL )
      AND (CompleteSet.NumberSeller IS NULL) AND (CompleteSet.NumberManufact IS NULL) AND (CompleteSet.CmplNumber IS NULL)
  --
  IF EXISTS (SELECT * FROM Displacement WHERE (Component = @EL) AND (Receipt = 1))
  DELETE Displacement WHERE (Component = @EL) AND (Receipt = 1)
  --  
  INSERT INTO Displacement (Component, Moving, Device, Workplace, Department, Receipt)
    SELECT CompleteSet.IdComponent, 
           ISNULL([Document].DocDate, '19961231'), 
           ISNULL('. ' + CAST(CompleteSet.DocumentPosition AS VARCHAR), '')
             + ISNULL( ', (' + CAST(CompleteSet.Price AS VARCHAR) + '.)', ''), 
           DocKind.Name + ISNULL( '  ' + [Document].DocNumber, ''), 
           ISNULL(NameTrademark, '<>'), 1
    FROM CompleteSet 
      INNER JOIN [Document] ON CompleteSet.DocumentId = [Document].IdDocument 
      INNER JOIN DocKind ON [Document].DocKind = DocKind.ID 
      LEFT OUTER JOIN FirmNamePower ON [Document].DocClient = FirmNamePower.ID
  WHERE (CompleteSet.IdComponent = @EL )
--
GO
--=====================================================================================================================
CREATE PROCEDURE RepairElementModelKind
AS
  UPDATE CompleteSet SET ModelKind = Model.ModlKind
    FROM CompleteSet 
      INNER JOIN Model ON CompleteSet.CmplIdModel = Model.IdModel
--
GO
--=====================================================================================================================
CREATE TRIGGER DisplaceHard ON CompleteSet FOR INSERT, UPDATE
AS
  SET NOCOUNT ON
  IF UPDATE ( CmplIdDevice )
    INSERT INTO Displacement (Component, Moving, DeviceID, Device, Workplace, Department)
      SELECT CompleteSet.IdComponent, GETDATE(), CompleteSet.CmplIdDevice, 
             ViewDevicePowerName.NameNumber, ViewWorkplacePowerName.NameUser, Department.DprtName
        FROM Inserted 
          INNER JOIN CompleteSet ON Inserted.IdComponent = CompleteSet.IdComponent
          INNER JOIN ViewDevicePowerName ON CompleteSet.CmplIdDevice = ViewDevicePowerName.ID 
          INNER JOIN ViewWorkplacePowerName ON ViewDevicePowerName.High = ViewWorkplacePowerName.ID 
          INNER JOIN Department ON ViewWorkplacePowerName.Higher = Department.IdDepartment
--
GO
--=====================================================================================================================
CREATE TRIGGER DisplaceDevice ON DeviceUnit FOR INSERT, UPDATE
AS
  SET NOCOUNT ON
  IF ( UPDATE ( DevcIdWorkplace ) OR UPDATE ( DevcIdKind ) OR UPDATE ( DevcName ) OR UPDATE ( DevcNumber ))
    INSERT INTO Displacement (Component, Moving, DeviceID, Device, Workplace, Department)
      SELECT CompleteSet.IdComponent, GETDATE(), DeviceUnit.IdDevice, 
             ViewDevicePowerName.NameNumber, ViewWorkplacePowerName.NameUser, Department.DprtName
        FROM Inserted 
          INNER JOIN DeviceUnit ON Inserted.IdDevice = DeviceUnit.IdDevice
          INNER JOIN CompleteSet ON DeviceUnit.IdDevice = CompleteSet.CmplIdDevice 
          INNER JOIN ViewDevicePowerName ON CompleteSet.CmplIdDevice = ViewDevicePowerName.ID 
          INNER JOIN ViewWorkplacePowerName ON ViewDevicePowerName.High = ViewWorkplacePowerName.ID 
          INNER JOIN Department ON ViewWorkplacePowerName.Higher = Department.IdDepartment
--
GO
--=====================================================================================================================
CREATE TRIGGER DisplaceWorkplace ON Workplace FOR INSERT, UPDATE
AS
  SET NOCOUNT ON
  IF ( UPDATE ( WrkpName ) OR UPDATE ([User]))
    INSERT INTO Displacement (Component, Moving, DeviceID, Device, Workplace, Department)
      SELECT CompleteSet.IdComponent, GETDATE(), DeviceUnit.IdDevice, 
             ViewDevicePowerName.NameNumber, ViewWorkplacePowerName.NameUser, Department.DprtName
        FROM Inserted 
          INNER JOIN Workplace ON Inserted.IdWorkplace = Workplace.IdWorkplace
          INNER JOIN DeviceUnit ON Workplace.IdWorkplace = DeviceUnit.DevcIdWorkplace 
          INNER JOIN CompleteSet ON DeviceUnit.IdDevice = CompleteSet.CmplIdDevice
          INNER JOIN ViewDevicePowerName ON CompleteSet.CmplIdDevice = ViewDevicePowerName.ID 
          INNER JOIN ViewWorkplacePowerName ON Workplace.IdWorkplace = ViewWorkplacePowerName.ID 
          INNER JOIN Department ON ViewWorkplacePowerName.Higher = Department.IdDepartment

--
GO
--=====================================================================================================================
CREATE TRIGGER DisplaceDepartment ON Department FOR INSERT, UPDATE
AS
  SET NOCOUNT ON
  IF UPDATE ( DprtName )
    INSERT INTO Displacement (Component, Moving, DeviceID, Device, Workplace, Department)
      SELECT CompleteSet.IdComponent, GETDATE(), DeviceUnit.IdDevice,
            ViewDevicePowerName.NameNumber, ViewWorkplacePowerName.NameUser, Department.DprtName
        FROM Inserted 
          INNER JOIN Department ON Inserted.IdDepartment = Department.IdDepartment
          INNER JOIN Workplace ON Department.IdDepartment = Workplace.WrkpIdDepartment 
          INNER JOIN DeviceUnit ON Workplace.IdWorkplace = DeviceUnit.DevcIdWorkplace 
          INNER JOIN CompleteSet ON DeviceUnit.IdDevice = CompleteSet.CmplIdDevice
          INNER JOIN ViewDevicePowerName ON CompleteSet.CmplIdDevice = ViewDevicePowerName.ID 
          INNER JOIN ViewWorkplacePowerName ON ViewDevicePowerName.High = ViewWorkplacePowerName.ID 
 --
 GO 
--=====================================================================================================================
-- 
--EXECUTE ExeElementMoveDocument 10149
--EXECUTE RepairElementModelKind