/*=====================================================================================================================

   " ".

    , .

=======================================================================================================================

  :

  03.03.2015 [+]    (  ).
  05.06.2013 [+] CloneDevice:  .
             [+] CloneComponents:  .
             [-] CloneCompSet.
  29.12.2012 [+] ViewDevicePowerName: .
  24.03.2012 [+] ViewDevicePowerName:   .
  21.03.2012 [+] ViewDevicePowerName:     .
  26.02.2012 [+] CloneCompSet: .
  24.02.2012 [*] .
  22.02.2012 [*] .
  16.11.2006 [+] GridDisplacement:  .
  15.11.2006 [+] GridKindConsolidation:    .
             [+]  .
  09.10.1997 [!]  .

=======================================================================================================================
  Copyright 19972015 TSK Developer Laboratory, Moscow.                                         Konstantin Tsvetkov
                                                                                                  e-mail: dev@tsklab.ru
                                                                                                  http://www.tsklab.ru
  CVS: $Id: scDevice.sql,v 1.7 2015/03/03 13:41:45 tsk Exp $
=====================================================================================================================*/
USE ComputerWare
--
EXECUTE CheckDropView       'LookupCompleteSet'
EXECUTE CheckDropView		    'ViewDevicePowerName'
EXECUTE CheckDropProcedure  'GridDisplacement'
EXECUTE CheckDropProcedure  'GridKindConsolidation'
EXECUTE CheckDropProcedure  'CloneComponents'
EXECUTE CheckDropProcedure  'CloneDevice'
EXECUTE CheckDropTrigger    'DeviceInsert'
EXECUTE CheckDropTrigger    'DeviceUpdate'
EXECUTE CheckDropTrigger    'DeviceDelete'
--
GO
--=====================================================================================================================
CREATE VIEW LookupCompleteSet
AS
  SELECT CompleteSet.IdComponent AS ID, 
         ISNULL((SELECT MAX(Moving) FROM Displacement WHERE (Component = CompleteSet.IdComponent)), 
                CompleteSet.DateSeller) AS Last, 
         ISNULL(DATEADD(m, CompleteSet.Warranty, Document.DocDate), GETDATE()) AS Warranty, 
         CompleteSet.DateManufact AS Manufact,
         ISNULL(DocKind.Name, '') 
           + '  ' + ISNULL(Document.DocNumber, '[/]') 
           + '  ' + ISNULL(CONVERT(VARCHAR(8), Document.DocDate, 4), '[.]') AS Doc, 
         ISNULL(Client.ClntNameFull, '[]') AS Client
    FROM CompleteSet 
      RIGHT OUTER JOIN Document ON Document.IdDocument = CompleteSet.DocumentId
      INNER JOIN DocKind ON DocKind.ID = Document.DocKind 
      LEFT OUTER JOIN Client ON Document.DocClient = Client.IdClient 
--
GO
--=====================================================================================================================
CREATE VIEW ViewDevicePowerName
AS
  SELECT  IdDevice                      AS ID,
          DevcIdWorkplace               AS High,
          KindOrder                     AS Sorting,
          case when ( KindName = '*')
            then ISNULL( DevcName, '[ ]')
            else KindName + ISNULL(', ' + DevcName,'')
          end                           AS NameKind,
          case when ( KindName = '*')
            then ISNULL( DevcName + ISNULL(', ' + [DisplayNumber], ''), ISNULL(' ' + [DisplayNumber], ''))
            else KindName + ISNULL(', ' + DevcName,'') + ISNULL(', ' + [DisplayNumber], '')
          end                           AS NameNumber,
          case when ( KindName = '*')
            then ISNULL( DevcName + ISNULL(' ' + [DisplayNumber], ''), ISNULL(' ' + [DisplayNumber], ''))
            else KindName + ISNULL(' ' + DevcName,'') + ISNULL(' ' + [DisplayNumber], '')
          end + ISNULL(' ' + DeviceUnit.Domain, '')  AS NameFind
    FROM DeviceUnit
      INNER JOIN Kind ON DeviceUnit.DevcIdKind = Kind.IdKind  
--
GO
--=====================================================================================================================
CREATE PROCEDURE GridKindConsolidation @TP CHAR(1) = 'H'
AS
  SELECT ID, Name, Short, [Order] FROM KindConsolidation WHERE (Type = @TP) ORDER BY [Order]
--
GO
--=====================================================================================================================
CREATE PROCEDURE GridDisplacement @CS INT
AS
  SELECT ID, Moving, Department + CHAR(13) + Workplace + CHAR(13) +  + Device AS Place
    FROM Displacement
    WHERE (Component = @CS)
    ORDER BY Moving DESC
--
GO
--=====================================================================================================================
CREATE PROCEDURE CloneComponents @ID INT
AS
  INSERT INTO CompleteSet (CmplIdDevice, CmplDateSet, CmplIdModel, CmplNumber, NumberManufact, DateManufact, NumberSeller, 
                           DateSeller, Warranty, DocumentId, DocumentPosition, Price)
    SELECT CmplIdDevice, GETDATE(), CmplIdModel, NULL, NULL, DateManufact, NumberSeller, DateSeller, Warranty, DocumentId, 
           DocumentPosition, Price
      FROM CompleteSet
      WHERE (IdComponent = @ID)
--
GO
--=====================================================================================================================
CREATE PROCEDURE CloneDevice @ID INT
AS
  INSERT INTO DeviceUnit (DevcIdWorkplace, DevcIdKind, DevcName, Chapter, DevcPrinting, DevcDescription, Used)
    SELECT DevcIdWorkplace, DevcIdKind, DevcName + ' (clone)', Chapter, DevcPrinting, DevcDescription, Used
      FROM DeviceUnit
      WHERE (IdDevice = @ID)
  --
  DECLARE @ND INT
  SET @ND = IDENT_CURRENT('DeviceUnit')
  INSERT INTO CompleteSet (CmplIdDevice, CmplDateSet, CmplIdModel)
    SELECT @ND, CmplDateSet, CmplIdModel
      FROM CompleteSet
      WHERE (CmplIdDevice = @ID)
--
GO
--=====================================================================================================================
CREATE TRIGGER DeviceInsert ON DeviceUnit FOR INSERT
AS
  SET NOCOUNT ON
  UPDATE Kind SET UsesDevice = (SELECT COUNT(*) FROM DeviceUnit WHERE ( DeviceUnit.DevcIdKind = inserted.DevcIdKind))
    FROM Kind INNER JOIN inserted ON Kind.IdKind = inserted.DevcIdKind
--
GO
--=====================================================================================================================
CREATE TRIGGER DeviceUpdate ON DeviceUnit FOR UPDATE
AS
  SET NOCOUNT ON
  UPDATE Kind SET UsesDevice = (SELECT COUNT(*) FROM DeviceUnit WHERE ( DeviceUnit.DevcIdKind = inserted.DevcIdKind))
    FROM Kind INNER JOIN inserted ON Kind.IdKind = inserted.DevcIdKind
  UPDATE Kind SET UsesDevice = (SELECT COUNT(*) FROM DeviceUnit WHERE ( DeviceUnit.DevcIdKind = deleted.DevcIdKind))
    FROM Kind INNER JOIN deleted ON Kind.IdKind = deleted.DevcIdKind
--
GO
--=====================================================================================================================
CREATE TRIGGER DeviceDelete ON DeviceUnit FOR DELETE
AS
  SET NOCOUNT ON
  UPDATE Kind SET UsesDevice = (SELECT COUNT(*) FROM DeviceUnit WHERE ( DeviceUnit.DevcIdKind = deleted.DevcIdKind))
    FROM Kind INNER JOIN deleted ON Kind.IdKind = deleted.DevcIdKind
--
GO
--=====================================================================================================================
-- 
--SELECT * FROM LookupCompleteSet
--SELECT * FROM ViewDevicePowerName
--EXECUTE GridKindConsolidation
--EXECUTE GridDisplacement 6090
--EXECUTE CloneComponents 10550
--EXECUTE CloneDevice 1447